# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	modules/NovellBcc.ycp
# Package:	Configuration of novell-bcc
# Summary:	NovellBcc settings, input and output functions
# Authors:	Ryan Okelberry <rokelberry@novell.com>
#
# $Id: NovellBcc.ycp 27914 2006-02-13 14:32:08Z locilka $
#
# Representation of the configuration of novell-bcc.
# Input and output routines.
require "yast"

module Yast
  class NovellBccClass < Module
    def main
      Yast.import "Pkg"
      Yast.import "UI"
      textdomain "novell-bcc"

      Yast.import "Label"
      Yast.import "Popup"
      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Service"
      Yast.import "SlideShow"
      Yast.import "Summary"
      Yast.import "Misc"
      Yast.import "NovellLDAP"
      Yast.import "NovellUtils"
      Yast.import "NovellPatterns"
      Yast.import "PackageSystem"
      Yast.import "Ncs"
      Yast.import "Wizard"

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Settings: Define all variables needed for configuration of bcc

      @Settings = {} # Holds configuration information
      @sysconf_file = "/etc/sysconfig/novell/bcc" #The /etc/sysconfig file.

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # DisplayMessage
    # @param [String] title Title to display (eg "Error")
    # @param [String] text Message to display
    def DisplayMessage(title, text)
      Popup.LongText(title, RichText(text), 60, 10)

      nil
    end

    # Prompt user for an existing directory
    # @param [String] prompt
    # @param [String] base location
    # @return [String] "" if cancelled, otherwise, an absolute path to an existing directory
    def PromptForDirectory(prompt, base)
      directoryPath = ""

      UI.OpenDialog(
        VBox(
          Label(prompt),
          HBox(
            TextEntry(Id(:dir), _("&Directory Path")),
            VCenter(PushButton(Id(:browse), _("&Browse...")))
          ),
          HBox(
            HStretch(),
            PushButton(Id(:ok), Opt(:default), Label.OKButton),
            PushButton(Id(:cancel), Label.CancelButton),
            HStretch()
          )
        )
      )

      UI.ChangeWidget(Id(:dir), :Value, base)
      UI.SetFocus(Id(:dir))

      input = nil
      begin
        input = UI.UserInput

        if input == :browse
          directoryPath = Convert.to_string(UI.QueryWidget(Id(:dir), :Value))

          result = UI.AskForExistingDirectory(
            directoryPath,
            _("Local Directory")
          )
          UI.ChangeWidget(Id(:dir), :Value, result) if result != nil
        elsif input == :ok
          directoryPath = Convert.to_string(UI.QueryWidget(Id(:dir), :Value))
        elsif input == :cancel
          directoryPath = ""
        end
      end until input == :cancel || input == :ok

      UI.CloseDialog

      directoryPath
    end

    # Check to see if NCS is installed
    # @return [Boolean] true if ncs is installed
    def NcsInstalled
      Pkg.TargetInit("/", false)

      if Pkg.IsProvided("novell-cluster-services")
        if "yes" ==
            Convert.to_string(
              SCR.Read(path(".sysconfig.ncs.SERVICE_CONFIGURED"))
            )
          if Service.Status("novell-ncs") == 0
            return true
          else
            Builtins.y2debug("NCS service is not running.")
          end
          return true
        else
          Builtins.y2debug("ncs packages installed, but not configured")
        end
      else
        Builtins.y2debug("ncs packages not installed.")
      end
      false
    end

    # Check to see if BCC package is installed
    # @return [Boolean] true if bcc packages are installed
    def BccPackagesInstalled
      found = false

      pattern_data = Pkg.ResolvableDependencies(
        Ops.get_string(@Settings, "PATTERN_NAME", ""),
        :pattern,
        ""
      )

      Builtins.foreach(pattern_data) do |pattern|
        if Ops.get_string(@Settings, "PATTERN_NAME", "") ==
            Ops.get_string(pattern, "name", "") &&
            :installed == Ops.get_symbol(pattern, "status", :unknown)
          found = true
          raise Break
        end
      end 


      found
    end

    # Popup to prompt is user would like to install NCS
    # @return true if yes is pressed
    def PromptInstallNcs
      Popup.AnyQuestion(
        "",
        "In order to continue, OES Cluster Services (NCS) must be installed.\n" +
          "\n" +
          "Would you like to install NCS ?",
        Label.YesButton,
        Label.NoButton,
        :yes
      )
    end

    # Popup to prompt is user would like to install NCS
    # @return true if yes is pressed
    def PromptInstallBcc
      Popup.AnyQuestion(
        "",
        "The required software packages for OES Business Continuity Cluster are not installed.\nWould you like to install them?",
        Label.YesButton,
        Label.NoButton,
        :yes
      )
    end

    # Method to lay down bcc rpms.  It will prompt the user for the path the installation rpms.
    # @return [Boolean] true if install went ok
    def InstallBcc
      NovellPatterns.InstallPattern(
        Ops.get_string(@Settings, "PATTERN_NAME", "")
      )
    end

    # Method to call ncs installation
    # @return [Boolean] true if ncs installation returns correctly
    def InstallNcs
      ret = WFM.call("ncs", [])
      ret == :next
    end

    # Read bcc settings from sysconfig file (if it exists)
    # @return [Boolean] true if the system should continue.  The only way this returns
    #		false is if the user chooses to not continue if previous settings were found
    def ReadCurrentBccSettings
      pattern_name = Convert.to_string(
        SCR.Read(path(".sysconfig.novell.bcc.PATTERN_NAME"))
      )
      pattern_name = "novell-bcc" if nil == pattern_name || "" == pattern_name
      Ops.set(@Settings, "PATTERN_NAME", pattern_name)

      config_type = Convert.to_string(
        SCR.Read(path(".sysconfig.novell.bcc.CONFIG_BCC_CONFIG_TYPE"))
      )
      config_type = "New" if nil == config_type || "" == config_type
      Ops.set(@Settings, "CONFIG_BCC_CONFIG_TYPE", config_type)

      start_now = Convert.to_string(
        SCR.Read(path(".sysconfig.novell.bcc.CONFIG_BCC_START_NOW"))
      )
      start_now = "yes" if nil == start_now || "" == start_now
      Ops.set(@Settings, "CONFIG_BCC_START_NOW", start_now)

      cluster_dn = Convert.to_string(
        SCR.Read(path(".sysconfig.ncs.CONFIG_NCS_CLUSTER_DN"))
      )
      if nil == cluster_dn || "" == cluster_dn
        Popup.Error(
          _(
            "It appears that OES Cluster Services (NCS) is not properly configured.  Please configure NCS."
          )
        )
        return false
      end
      Ops.set(@Settings, "CONFIG_BCC_CLUSTER_DN", cluster_dn)

      server_list = Builtins.splitstring(
        Convert.to_string(
          SCR.Read(path(".sysconfig.ncs.CONFIG_NCS_LDAP_SERVERS"))
        ),
        ","
      )
      if 0 == Builtins.size(server_list)
        Popup.Error(
          _(
            "It appears that OES Cluster Services (NCS) is not properly configured.  Please configure NCS."
          )
        )
        return false
      end
      Ops.set(@Settings, "CONFIG_LDAP_SERVER_LIST", server_list)

      Ops.set(
        @Settings,
        "SERVICE_CONFIGURED",
        Convert.to_string(
          SCR.Read(path(".sysconfig.novell.bcc.SERVICE_CONFIGURED"))
        )
      )
      if "yes" == Ops.get_string(@Settings, "SERVICE_CONFIGURED", "no")
        if !Popup.YesNo(
            _(
              "It appears that OES Business Continuity Cluster has already been configured.\n" +
                "\n" +
                "Would you like to continue anyway?"
            )
          )
          return false
        end
      end

      true
    end

    # upgrade eDir objects from 1.0 to 1.1
    def Modify_eDir
      # TODO convert nsmi scripts to smi-s or store in format for use by our nsmi provider and store on resource
      # TODO remove nsmi info from pool objects
      # TODO ???
      true
    end

    # Find the location of a file
    #
    # @param [Array<String>] directories List of directories to look for file (non-recursive).
    # @param [String] filename name of file
    # @return [String] nil if file was not found, otherwise the absolute path
    def FindFile(directories, filename)
      directories = deep_copy(directories)
      absPath = nil

      Builtins.foreach(directories) do |directory|
        if 0 ==
            Convert.to_integer(
              SCR.Execute(
                path(".target.bash"),
                Builtins.sformat("ls %1/%2", directory, filename)
              )
            )
          absPath = Ops.add(Ops.add(directory, "/"), filename)
        end
      end 


      return absPath if nil != absPath

      fromUser = UI.AskForExistingFile(
        Ops.get(directories, 0, ""),
        filename,
        Builtins.sformat(
          _("Please provide the location of the file '%1'..."),
          filename
        )
      )

      if nil == fromUser || nil == Builtins.findlastof(fromUser, filename)
        return nil
      else
        return fromUser
      end
    end


    def ExtendSchema
      extended = false
      error = ""
      admin = NovellUtils.Ldap2Nds(
        Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      )
      password = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )
      sch_file = "/opt/novell/bcc/schema/bcc.sch"
      ndssch_file = FindFile(
        ["/usr/bin", "/opt/novell/eDirectory/bin"],
        "ndssch"
      )

      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
      )
      selected_server_list = Ops.get_list(
        @Settings,
        "CONFIG_LDAP_SERVER_LIST",
        []
      )
      rmap = nil

      Builtins.foreach(selected_server_list) do |server_ip|
        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(Ops.add(Ops.add(ndssch_file, " -h "), server_ip), " "),
                  "\""
                ),
                admin
              ),
              "\""
            ),
            " "
          ),
          sch_file
        )
        Builtins.y2debug("%1", cmd)
        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add("ADM_PASSWD=", password), " "), cmd)
          )
        )
        if Ops.get_integer(rmap, "exit", -1) == 0
          extended = true
          raise Break
        end
        error = Ops.add(
          Ops.add(error, Ops.get_string(rmap, "stderr", "")),
          "\n"
        )
      end 


      if !extended
        DisplayMessage(
          _("Error"),
          Ops.add(_("Failed extending eDirectory schema:\n"), error)
        )
      end

      extended
    end

    # boolean InstallMof()
    # {
    # 	map ret = (map)SCR::Execute(.target.bash_output, "/usr/bin/sfcbstage -r /opt/novell/bcc/mof/bcc.reg /opt/novell/bcc/mof/bcc.mof");
    #
    # 	if (ret["exit"]:1 != 0)
    # 	{
    # 		DisplayMessage(_("Error"), _("Failed extending CIMOM schema:<br>") +ret["stderr"]:"");
    # 		return false;
    # 	}
    #
    # 	ret = (map)SCR::Execute(.target.bash_output, "/usr/bin/sfcbrepos -f");
    #
    # 	if (ret["exit"]:1 != 0)
    # 	{
    # 		DisplayMessage(_("Error"), _("Failed extending CIMOM schema:<br>") +ret["stderr"]:"");
    # 		return false;
    # 	}
    #
    # 	return true;
    # }


    def EnableBCC
      Service.Enable("novell-bcc")
    end


    def StartSFCB
      Service.Start("sblim-sfcb") && Service.Status("sblim-sfcb") == 0
    end


    def StartBCC
      Service.Start("novell-bcc") && Service.Status("novell-bcc") == 0
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # Read all novell-bcc settings
    # @return true on success
    def Read
      # Bcc read dialog caption
      caption = _("OES Business Continuity Cluster (BCC) Configuration")

      steps = 3

      sl = 500
      Builtins.sleep(sl)

      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          _("Check for installation of OES Cluster Services (NCS)"),
          _("Read OES Business Continuity Cluster (BCC) settings"),
          _("Check for installation of software packages")
        ],
        [
          _("Check for installation of OES Cluster Services (NCS)"),
          _("Read OES Business Continuity Cluster (BCC) settings"),
          _("Check for installation of software packages")
        ],
        ""
      )

      # check that ncs is installed
      return false if Abort()
      Progress.NextStage
      if !NcsInstalled()
        if PromptInstallNcs()
          if !InstallNcs()
            DisplayMessage(
              _("Error"),
              _(
                "OES Cluster Serices (NCS) was not installed.\n" +
                  "\n" +
                  "Aborting installation."
              )
            )
            return false
          end
        else
          DisplayMessage(
            _("Error"),
            _(
              "OES Cluster Serices (NCS) must be installed before you can continue.\n" +
                "\n" +
                "Aborting installation."
            )
          )
          return false
        end
      end

      @Settings = {}

      # read bcc settings
      return false if Abort()
      Builtins.sleep(1000)
      Progress.NextStage
      if !ReadCurrentBccSettings()
        DisplayMessage(_("Warning"), _("Aborting configuration."))
        return false
      end

      # check that bcc software is installed
      return false if Abort()
      Progress.NextStage
      if !BccPackagesInstalled()
        if PromptInstallBcc()
          if !InstallBcc()
            DisplayMessage(
              _("Error"),
              _(
                "The required packages could not be installed.\n" +
                  "\n" +
                  "Aborting installation."
              )
            )
            return false
          end
        else
          DisplayMessage(
            _("Error"),
            _(
              "OES Business Continuity Cluster software must be installed before continuing.\n" +
                "\n" +
                "Aborting installation."
            )
          )
          return false
        end
      end

      return false if Abort()
      # Progress finished
      Progress.NextStage
      Builtins.sleep(sl)


      NovellLDAP.Read

      return false if Abort()
      @modified = false
      true
    end

    # Write all novell-bcc settings
    # @return true on success
    def Write
      newInstall = Ops.get_string(@Settings, "CONFIG_BCC_CONFIG_TYPE", "New") == "New"
      startNow = Ops.get_string(@Settings, "CONFIG_BCC_START_NOW", "yes") == "yes"

      # Bcc read dialog caption
      caption = _(
        "Saving OES Business Continuity Cluster (BCC) Configuration"
      )

      stages = [_("Save Local Settings")]
      stages = Builtins.add(stages, _("Extend eDirectory Schema"))

      if !newInstall
        stages = Builtins.add(stages, _("Upgrade Existing eDirectory objects"))
      end

      stages = Builtins.add(stages, _("Extend local CIM object base"))
      stages = Builtins.add(stages, _("Enable BCC Services"))

      stages = Builtins.add(stages, _("Start BCC Services")) if startNow

      steps = Builtins.size(stages)

      sl = 1000

      Progress.New(caption, " ", steps, stages, stages, "")

      # write settings
      return false if Abort()
      Progress.NextStage
      SCR.Write(
        path(".sysconfig.novell.bcc.CONFIG_BCC_CONFIG_TYPE"),
        Ops.get_string(@Settings, "CONFIG_BCC_CONFIG_TYPE", "New")
      )
      SCR.Write(
        path(".sysconfig.novell.bcc.CONFIG_BCC_START_NOW"),
        Ops.get_string(@Settings, "CONFIG_BCC_START_NOW", "yes")
      )

      # extend schema
      return false if Abort()
      Progress.NextStage
      return false if !ExtendSchema()
      Builtins.sleep(sl)

      # upgrade edir objects
      if !newInstall
        return false if Abort()
        Progress.NextStage
        return false if !Modify_eDir()
        Builtins.sleep(sl)
      end

      # install mof
      #     if(Abort()) return false;
      #     Progress::NextStage ();
      # 	if (!InstallMof()) {
      # 		return false;
      # 	}
      # 	sleep(sl);

      # enable bcc (chkconfig)
      return false if Abort()
      Progress.NextStage
      if !EnableBCC()
        DisplayMessage(
          _("Warning"),
          _(
            "Problem enabling BCC services for startup.\n" +
              "\n" +
              "Check /var/log/YaST2/y2log for errors."
          )
        ) 
        # continue
      end
      Builtins.sleep(sl)

      # start sfcb
      return false if Abort()
      Progress.NextStage
      if !StartSFCB()
        DisplayMessage(
          _("Warning"),
          _(
            "Problem starting CIMOM services for startup.\n" +
              "\n" +
              "Check /var/log/YaST2/y2log for errors."
          )
        ) 
        # continue
      end
      Builtins.sleep(sl)

      # start bcc
      if startNow
        return false if Abort()
        Progress.NextStage
        if !StartBCC()
          DisplayMessage(
            _("Warning"),
            _(
              "Problem starting BCC services.\n" +
                "\n" +
                "Check /var/log/YaST2/y2log for errors."
            )
          )
          # continue;
          Builtins.sleep(sl)
        end
      end

      Progress.Finish
      Builtins.sleep(sl)

      SCR.Write(path(".sysconfig.novell.bcc.SERVICE_CONFIGURED"), "yes")

      return false if Abort()
      true
    end

    # Get all novell-bcc settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      missingTags = []
      Ops.set(
        @Settings,
        "CONFIG_BCC_CONFIG_TYPE",
        Ops.get_string(settings, "config-type", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_BCC_START_NOW",
        Ops.get_string(settings, "start-now", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_BCC_CLUSTER_DN",
        Ops.get_string(settings, "cluster-dn", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LDAP_SERVER_LIST",
        Builtins.splitstring(Ops.get_string(settings, "ldap-server", ""), ",")
      )
      Ops.set(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        Ops.get_string(settings, "admin-dn", "")
      )
      Ops.set(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        Ops.get_string(settings, "admin-password", "")
      )

      if Ops.get_string(settings, "config-type", "") == ""
        Ops.set(missingTags, Builtins.size(missingTags), "config-type")
      end

      if Ops.get_string(settings, "start-now", "") == ""
        Ops.set(missingTags, Builtins.size(missingTags), "start-now")
      end

      if Ops.get_string(settings, "cluster-dn", "") == ""
        Ops.set(missingTags, Builtins.size(missingTags), "cluster-dn")
      end

      if Ops.get_string(settings, "ldap-server", "") == ""
        Ops.set(missingTags, Builtins.size(missingTags), "ldap-server")
      end

      if Ops.get_string(settings, "admin-dn", "") == ""
        Ops.set(missingTags, Builtins.size(missingTags), "admin-dn")
      end

      if Ops.get_string(settings, "admin-password", "") == ""
        Ops.set(missingTags, Builtins.size(missingTags), "admin-password")
      end

      if 0 != Builtins.size(missingTags)
        error = _(
          "The following tags are either blank or missing in the control file:"
        )

        Builtins.foreach(missingTags) do |tag|
          error = Ops.add(Ops.add(Ops.add(error, " '"), tag), "'")
        end 

        error = Ops.add(
          error,
          _(".  Please make the necessary corrections and try again.")
        )
        DisplayMessage(_("Error"), error)
        return false
      end
      true
    end

    # Dump the novell-bcc settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      Ops.set(
        settings,
        "config-type",
        Ops.get_string(@Settings, "CONFIG_BCC_CONFIG_TYPE", "")
      )
      Ops.set(
        settings,
        "start",
        Ops.get_string(@Settings, "CONFIG_BCC_START_NOW", "")
      )

      deep_copy(settings)
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      # TODO FIXME: your code here...
      # Configuration summary text for autoyast
      [_("Configuration summary..."), []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :Settings, :type => "map"
    publish :function => :DisplayMessage, :type => "void (string, string)"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  NovellBcc = NovellBccClass.new
  NovellBcc.main
end
